-- Bailer et al.[1] (fast arch)
import 'fastpatch'

imH = 864 --image height
imW = 576 --image width

if #arg > 1 then
  imH = arg[1] 
  imW = arg[2] 
end

pH = 56 -- patch height
pW = 56 -- patch width

sL1 = 2 -- stride, 1. pooling layer
sL2 = 2 -- stride, 2. pooling layer


outChans = 256 -- output channels (last layer)

-- our code can currently only handle image sizes that are multiples of sL1*sL2, so make sure it is:
imW = math.ceil(imW /(sL1*sL2) )*sL1*sL2
imH = math.ceil(imH /(sL1*sL2) )*sL1*sL2
print("The image size for benchmarking is (might contain padding): Height: " ..  imH .. " Width: ".. imW)


-- The patch network (C_P):
features = 64
net1 =  nn.Sequential()
net1:add(nn.SpatialConvolution(3, features, 5, 5))
net1:add(nn.SpatialMaxPooling(2,2,2,2))
net1:add(nn.Tanh())
net1:add(nn.SpatialConvolution(features, features*1.25, 5, 5)) 
net1:add(nn.Tanh())
net1:add(nn.SpatialConvolution(features*1.25, features*2, 5, 5)) 
net1:add(nn.SpatialMaxPooling(2,2,2,2))
net1:add(nn.Tanh())
net1:add(nn.SpatialConvolution(features*2, features*3, 5, 5))
net1:add(nn.Tanh())
net1:add(nn.SpatialConvolution(features*3, features*4, 5, 5))
net1:add(nn.Tanh())
net1:add(nn.SpatialConvolution(features*4, features*4, 1, 1))
net1 = net1:cuda()


-- The network for the whole image (C_I):
net2 =  nn.Sequential()
net2:add(multiPoolPrepare(pH,pW))  
net2:add(net1.modules[1])
net2:add(multiMaxPooling(sL1,sL1,sL1,sL1))
net2:add(net1.modules[3])
net2:add(net1.modules[4])
net2:add(net1.modules[5])
net2:add(net1.modules[6])
net2:add(multiMaxPooling(sL2,sL2,sL2,sL2))
net2:add(net1.modules[8])
net2:add(net1.modules[9])
net2:add(net1.modules[10])
net2:add(net1.modules[11])
net2:add(net1.modules[12])
net2:add(net1.modules[13])

net2:add(unwrapPrepare())
net2:add(unwrapPool(outChans, imH/(sL1*sL2), imW/(sL1*sL2),sL2,sL2))
net2:add(unwrapPool(outChans, imH/sL1, imW/sL1 , sL1, sL1) )
net2:add(nn.View(-1,imH,imW)) 
net2 = net2:cuda()

-- benchmark C_I:
testImage = torch.rand(3,imH,imW):cuda()
sys.tic()
net2:forward(testImage)
t = sys.toc()
print("Total Time for C_I: ", t)


-- benchmark C_P:
-- Note: For simplcity we do not extract our testpatches from "testImage", but generate new random data. For a speed benchmark this does not make a difference.
-- It also allows us more easily to only benchmark the raw network execution time (see l. 242).
testPatches = torch.rand(imH,3,pH,pW):cuda()  -- we create imH patches 

sys.tic()
for i=1,imW do -- we execute the imH patches imW times to get imW*imH patches.
  net1:forward(testPatches)
end
t = sys.toc()
print("Total Time for C_P: ", t)
