import 'nn'
import 'cunn'

-- Adds padding to the image and adds the dimesion M
function multiPoolPrepare(patchY,patchX)
 padx = patchX-1
 pady = patchY-1
 local net = nn.Sequential()
 net:add(nn.Padding(2,-torch.ceil(pady/2)))
 net:add(nn.Padding(3,-torch.ceil(padx/2)))
 net:add(nn.Padding(2,torch.floor(pady/2)))
 net:add(nn.Padding(3,torch.floor(padx/2)))
 net:add(nn.Unsqueeze(1))
 return net
end

-- Layer prepares unwraping. Added after the network
function unwrapPrepare()
 local net = nn.Sequential()
 net:add(nn.View(-1):setNumInputDims(3)) --puts dim 2,3,4 to one dim
 net:add(nn.Transpose( {1,2} ))
 return net
end

-- The actual unwraping. See example for usage
-- curImg = imgSize /(all still existing strides)
function unwrapPool(outChans, curImgW, curImgH, dW, dH)
  local net = nn.Sequential()
  net:add(nn.View(outChans,curImgH,curImgW,dH,dW,-1))
  net:add(nn.Transpose({3,4})) -- {3,4} not {2,3} as k is first dim 
  return net
end

-- Replaces normal maxpooling
function multiMaxPooling(kW,kH,dW,dH)
  local c1 = nn.DepthConcat(1)
  for i = 0,dH-1 do
    for j = 0,dW-1 do
        c1:add(nn.SpatialMaxPooling(kW,kH,dW,dH,-j,-i))
    end
  end
  return c1
end

-- Replaces strided convolution.
function multiConv(nInputPlane, nOutputPlane, kW, kH, dW, dH)
  local c1 = nn.DepthConcat(1)
  for i = 0,dH-1 do
    for j = 0,dW-1 do
        c1:add(nn.SpatialConvolution(nInputPlane, nOutputPlane, kW, kH, dW, dH,-j,-i))
    end
  end
  return c1
end

